% beispiele.m
% Kapitel 6: Signalverarbeitung - Signal Processing Toolbox
%            Beispiele zum Buch

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% 6.1 Interpolation, Approximation und Abtastung
%      interp1

x        = -0.6:2.4;
y        = exp (-x.^2/2);
x_interp = -1:0.1:3;
y_interp_linear = interp1 (x, y, x_interp, 'linear');
y_interp_cubic  = interp1 (x, y, x_interp, 'cubic');
y_interp_spline = interp1 (x, y, x_interp, 'spline');

figure
    plot (x, y, 'ko')
    hold on
    plot (x_interp, exp (-x_interp.^2/2), 'k:')
    plot (x_interp, y_interp_linear, 'b-.')
    plot (x_interp, y_interp_cubic,  'g--')
    plot (x_interp, y_interp_spline, 'r-')
    axis ([-1 3 0 1.1])
    legend ('Testpunkte', 'ideal', 'linear', 'kubisch', 'Splines')
    title ('interp1')

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% griddata, meshgrid

x = rand (100, 1) * 4 - 2;                 % Erzeugt Zufallszahlen-Vektor
y = rand (100, 1) * 4 - 2;

z = x .* exp (-x.^2 - y.^2);               % Testpunkte

[XI, YI] = meshgrid (-2:.25:2, -2:.25:2);  % Koordinaten-Raster
ZI = griddata (x, y, z, XI, YI, 'v4');     % Raster aus Testpunkten

figure
    mesh (XI, YI, ZI)                          % Raster zeichnen
    hold on
    plot3 (x, y, z, 'o')                       % Testpunkte zeichnen
    title ('griddata')

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Basic Fitting Tool

figure
x = -0.6:2.4;
y = exp (-x.^2/2);
plot (x, y, 'ko')

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% upsample, downsample

t = 0.025:0.025:1;                           % Zeitvektor
x = 4*sin (2*pi*t) + cos (pi/4 + 16*pi*t);   % Datenvektor ("Messwerte")
y = downsample (x, 4);                       % Reduzierung der Abtastrate
z = upsample (y, 4);                         % Erhhung der Abtastrate
figure
subplot (131)
    plot ([1:4:40], y, 'ro')
    hold on
    plot (x, 'k.-')
    xlabel ('Samples')
    ylabel ('Amplitude')
    title ('Originalsignal')
subplot (132)
    plot (y, 'ro')
    hold on
    plot (y, 'k.-')
    xlabel ('Samples')
    title ('Abtastrate reduziert')
subplot (133)
    plot ([1:4:40], y, 'ro')
    hold on
    plot (z, 'k.-')
    xlabel ('Samples')
    title ('Abtastrate wieder erhht')

%% interp, decimate

y = decimate (x, 4, 8, 'fir');  % Reduzierung der Abtastrate, FIR-Filter 8. Ord.
z = interp (y, 4);              % Erhhung der Abtastrate
figure
subplot (131)
    plot ([1:4:40], y, 'ro')
    hold on
    plot (x, 'k.-')
    xlabel ('Samples')
    ylabel ('Amplitude')
    title ('Originalsignal')
subplot (132)
    plot (y, 'ro')
    hold on
    plot (y, 'k.-')
    xlabel ('Samples')
    title ('Abtastrate reduziert')
subplot (133)
    plot ([1:4:40], y, 'ro')
    hold on
    plot (z, 'k.-')
    xlabel ('Samples')
    title ('Abtastrate wieder erhht')

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% 6.2 Spektralanalyse

t = 0.01:0.01:0.5;                           % Zeitvektor (Fs = 100 Hz)
x = 5 + 8*sin(2*pi*8*t) + 4*cos(2*pi*33*t);  % Datenvektor ("Messwerte")

Ts = diff (t(1:2));                          % Abtastzeit
N = length (x);                              % Lnge des Datenvektors
f = [0:floor((N-1)/2)] / (N*Ts);             % Frequenzvektor fr Plot

X = fft (x);                                 % Fouriertransformation
X = X / N;                                   % Normierung
X = [X(1) 2*X(2:floor((N-1)/2)+1)];          % Begrenzen auf < F_max

figure
subplot (211)
    plot (t, x, '.-')
    axis ([0 0.5 -15 25])
    xlabel ('Zeit [s]')
    title ('Signal')
subplot (212)
    stem (f, abs (X))
    axis ([0 50 0 10])
    xlabel ('Frequenz [Hz]')
    ylabel ('Amplitude')
    title ('Spektrum')

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Averaging

t = 0.01:0.01:25;                            % Zeitvektor (Fs = 100 Hz)
x = 5 + 8*sin(2*pi*8*t) + 4*cos(2*pi*33*t);  % Datenvektor ("Messwerte")
x = x + 5*(randn (1,length(t)));             % Rauschen hinzufgen

Ts = diff (t(1:2));                          % Abtastzeit
N_seq = 50;                                  % Lnge einer Sequenz
N = length (x);                              % Lnge des Datenvektors
f = [0:N_seq/2-1] / (N_seq*Ts);              % Frequenzvektor fr Plot

X0 = abs (fft (x (1:N_seq)));                % Spektrum der 1. Sequenz
X0 = [X0(1) 2*X0(2:N_seq/2)] / N_seq;        % Begrenzen, Normieren

X = zeros (1, N_seq);                        % Leeres Array erzeugen
for k = 1:N/N_seq
    x_seq = x (1+(k-1)*N_seq:k*N_seq);       % Sequenz ausschneiden
    X = X + abs (fft (x_seq));               % DFT und Summation
end
X = [X(1) 2*X(2:N_seq/2)] / N;               % Begrenzen, Normieren

figure
subplot (211)
    stem (f, X0)
    axis ([0 50 0 10])
    xlabel ('Frequenz [Hz]')
    ylabel ('Amplitude')
    title ('Spektrum ohne Averaging')
subplot (212)
    stem (f, X)
    axis ([0 50 0 10])
    xlabel ('Frequenz [Hz]')
    ylabel ('Amplitude')
    title ('Spektrum mit 50fachem Averaging')

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Fensterung des Datensatzes

recht   = rectwin(21); recht = [0; recht; 0];
recht_x = 1:1:21; recht_x = [1 recht_x 21];
ham     = hamming(21);
bart    = bartlett(21);
black   = blackman(21);
ha      = hann(21);
hang    = hanning(21);
tri     = triang(21);

figure
    plot (recht_x, recht, 'k-')
    hold on
    plot (tri, 'b-')
    plot (ham, 'c-')
    plot (ha, 'm-')
    plot (black, 'r-')
    axis ([0 22 0 1.2])
    legend ('Rechteck', 'Dreieck', 'Hamming', 'Hann', 'Blackman', 'Location', 'S')
    xlabel ('n'); ylabel ('F(n)'); title ('Fensterfunktionen')

%% Beispiel: Anwendung auf DFT

t = 0.01:0.01:0.5;                           % Zeitvektor (Fs = 100 Hz)
x = 8*sin(2*pi*11*t) + 4*cos(2*pi*33*t);     % Datenvektor ("Messwerte")

Ts = diff (t(1:2));                          % Abtastzeit
N = length (x);                              % Lnge des Datenvektors
f = [0:floor((N-1)/2)] / (N*Ts);             % Frequenzvektor fr Plot

X = fft (hamming (N)' .* x);                 % Fouriertransformation
X = X / N;                                   % Normierung
X = [X(1) 2*X(2:floor((N-1)/2)+1)];          % Begrenzen auf < F_max

X_ohne = fft (x);                            % Fouriertransformation
X_ohne = X_ohne / N;                         % Normierung
X_ohne = [X_ohne(1) 2*X_ohne(2:floor((N-1)/2)+1)];  % Begrenzen auf < F_max

figure
subplot (121)
    stem (f, abs (X_ohne))
    axis ([0 50 0 10])
    xlabel ('Frequenz [Hz]')
    ylabel ('Amplitude')
    title ('Spektrum ohne Fensterung')
subplot (122)
    stem (f, abs (X))
    axis ([0 50 0 10])
    xlabel ('Frequenz [Hz]')
    ylabel ('Amplitude')
    title ('Spektrum mit Hamming-Fenster')
   
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Leistungsdichtespektrum

t = 0.01:0.01:25;                               % Zeitvektor (Fs = 100 Hz)
x = 2 + 8*sin(2*pi*8*t) + 4*cos(2*pi*33*t);     % Datenvektor ("Messwerte")
x = x + 5*(randn (1,length(t)));                % Rauschen hinzufgen

figure
    [Pxx, F] = pwelch (x, rectwin(250), [], 250, 1/diff(t(1:2)));
    plot (F, 10*log10(Pxx))
    axis ([0 50 -10 20])
    xlabel ('Frequenz [Hz]')
    ylabel ('Leistungsdichte [dB/Hz]')
    title ('Spektrum mit "pwelch"')

%% Spektralanalyse nicht-periodischer Signale
    
t = 0.01:0.01:25;                       % Zeitvektor (F = 100 Hz)
x = 5 + 8  * sin (2*pi*(8+t).*t) + ...  % Datenvektor (frequenzvariabler Anteil)
        t .* cos (2*pi*33*t);           %             (amplitudenvariabler Anteil)
figure
    spectrogram (x, 125, [], 125, 100)	% Spektralanalyse mit spectrogram

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% 6.3 Korrelationsfunktionen
%      Beispiel zur Autokorrelation

Fs          = 44.1e3;                    % Abtastfrequenz [Hz]
Ts          = 1/Fs;                      % Abtastzeit
t           = 0:Ts:0.5;                  % Zeitvektor
N           = length (t);                % Lnge des Datensatzes
signal      = randn (1, N);              % Nutzsignal als Zufallszahlen erzeugen

echo1       = zeros (1, N);              % Verzgertes und abgeschwchtes Echo 1
delay1      = 30e-3;                     % Verzgerung Echo 1 [s]
delayIndex1 = round (delay1 / Ts);       % Index-Verschiebung Echo 1
echo1 (delayIndex1+1:end) = 0.5 * signal (1:end-delayIndex1);

echo2       = zeros (1, N);              % Verzgertes und abgeschwchtes Echo 2
delay2      = 100e-3;                    % Verzgerung Echo 2 [s]
delayIndex2 = round (delay2 / Ts);       % Index-Verschiebung Echo 2
echo2 (delayIndex2+1:end) = 0.1 * signal (1:end-delayIndex2);

compound = signal + echo1 + echo2;       % zusammengesetztes Signal
cxx = xcorr (compound, 'unbiased');      % Autokorrelation

figure
subplot(211)
    plot (t(1:221), compound(1:221))
    title ('Signal im Zeitbereich (Ausschnitt)')
    xlabel ('Zeit [s]')
    axis([0 0.005 -3 3])
subplot(212)
    plot (t, cxx (N:2*N-1))              % Nur positive Zeitachse plotten
    axis ([-0.02 0.17 -0.2 1.4])
    title ('Autokorrelation (unbiased)')
    xlabel ('Verschiebung [s]')

%% Beispiel zur Kreuzkorrelation

Fs     = 10e3;                                   % Abtastfrequenz
Ts     = 1/Fs;                                   % Abtastzeit
t      = 0:Ts:0.5;                               % Zeitvektor
N      = length (t);                             % Lnge des Datensatzes

seq1   = sign (randn (1, N));                    % Zufallsfolge Sender 1
seq2   = sign (randn (1, N));                    % Zufallsfolge Sender 2
seq3   = sign (randn (1, N));                    % Zufallsfolge Sender 3
seq4   = sign (randn (1, N));                    % Zufallsfolge Sender 4

delay1 = 100e-3;                                 % Laufzeit von Sender 1
delay2 = 200e-3;                                 % Laufzeit von Sender 2
delay3 = 150e-3;                                 % Laufzeit von Sender 3
delay4 = 400e-3;                                 % Laufzeit von Sender 4

delayIndex1 = round (delay1 / Ts);               % Index-Verschiebung Empfang von Sender 1
delayIndex2 = round (delay2 / Ts);               % Index-Verschiebung Empfang von Sender 2
delayIndex3 = round (delay3 / Ts);               % Index-Verschiebung Empfang von Sender 3
delayIndex4 = round (delay4 / Ts);               % Index-Verschiebung Empfang von Sender 4

delayedSeq1 = [seq1(end-delayIndex1+1:end) seq1(1:end-delayIndex1)];
delayedSeq2 = [seq2(end-delayIndex2+1:end) seq2(1:end-delayIndex2)];
delayedSeq3 = [seq3(end-delayIndex3+1:end) seq3(1:end-delayIndex3)];
delayedSeq4 = [seq4(end-delayIndex4+1:end) seq4(1:end-delayIndex4)];

noise = randn (1, N);                            % Rauschen

compound = delayedSeq1 + ...                     % Summensignal
           delayedSeq2 + ...
           delayedSeq3 + ...
           delayedSeq4 + ...
           noise;

cxy1 = xcorr (compound, seq1, 'biased');         % Kreuzkorrelation Sender 1
cxy2 = xcorr (compound, seq2, 'biased');         % Kreuzkorrelation Sender 2
cxy3 = xcorr (compound, seq3, 'biased');         % Kreuzkorrelation Sender 3
cxy4 = xcorr (compound, seq4, 'biased');         % Kreuzkorrelation Sender 4

N_min = (length (cxy1) - 1) / 2 + 1;             % Min Index fr Plot
N_max = length (cxy1);                           % Max Index fr Plot

figure
plot (t, cxy1 (N_min:N_max), ...
      t, cxy2 (N_min:N_max), ...
      t, cxy3 (N_min:N_max), ...
      t, cxy4 (N_min:N_max))
axis ([0 0.5 -0.2 1])
legend ('S1, 100ms', 'S2, 200 ms', 'S3, 150ms', 'S4, 400 ms')
xlabel ('Laufzeit [s]')
title ('Kreuzkorrelation (biased)')

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% 6.4 Filter
%      Analoge Filter

[B, A] = butter (4, 1000*2*pi, 's');  % Butterworth-Tiefpass 4. Ord., Fg = 1 kHz
[H, W] = freqs (B, A);                % Frequenzgang und Frequenzvektor berechnen
semilogx (W/(2*pi), abs (H));         % Frequenzgang ausgeben

[z, p, k]    = cheby1 (6, 3, 1000*2*pi, 'high', 's');             % Hochpass
[a, b, c, d] = ellip  (6, 3, 20, [100 200]*2*pi, 'stop', 's');    % Bandsperre

fg       = 1000;  % [Hz]
W        = logspace (2, 4, 601) * 2*pi;
axisskal = [0.1 10 0.001 2];
skalax   = [0.1 1 10];
skalay   = -40:20:20;

figure
subplot (141)
    [B, A] = besself (2, fg*2*pi * 1.272);
    [H]    = freqs (B, A, W);
    loglog (W/2/pi/1000, abs (H), 'c-')
    hold on
    [B, A] = besself (4, fg*2*pi * 1.514);
    [H]    = freqs (B, A, W);
    loglog (W/2/pi/1000, abs (H), 'b-')
    [B, A] = besself (8, fg*2*pi * 1.932);
    [H]    = freqs (B, A, W);
    loglog (W/2/pi/1000, abs (H), 'k-')
    axis (axisskal)
    set (gca, 'xticklabel', skalax)
    set (gca, 'ytick', 10.^(skalay/20))
    set (gca, 'yticklabel', skalay)
    title ('Bessel (korrigiert)')
    xlabel ('Frequenz [kHz]')
    ylabel ('Amplitude')

subplot (142)
    [B, A] = butter (2, fg*2*pi, 's');
    [H]    = freqs (B, A, W);
    loglog (W/2/pi/1000, abs (H), 'c-')
    hold on
    [B, A] = butter (4, fg*2*pi, 's');
    [H]    = freqs (B, A, W);
    loglog (W/2/pi/1000, abs (H), 'b-')
    [B, A] = butter (8, fg*2*pi, 's');
    [H]    = freqs (B, A, W);
    loglog (W/2/pi/1000, abs (H), 'k-')
    axis (axisskal)
    set (gca, 'xticklabel', skalax)
    set (gca, 'ytick', 10.^(skalay/20))
    set (gca, 'yticklabel', skalay)
    title ('Butterworth')
    xlabel ('Frequenz [kHz]')

subplot (143)
    [B, A] = cheby1 (2, 3, fg*2*pi, 's');
    [H]    = freqs (B, A, W);
    loglog (W/2/pi/1000, abs (H), 'c-')
    hold on
    [B, A] = cheby1 (4, 3, fg*2*pi, 's');
    [H]    = freqs (B, A, W);
    loglog (W/2/pi/1000, abs (H), 'b-')
    [B, A] = cheby1 (8, 3, fg*2*pi, 's');
    [H]    = freqs (B, A, W);
    loglog (W/2/pi/1000, abs (H), 'k-')
    axis (axisskal)
    set (gca, 'xticklabel', skalax)
    set (gca, 'ytick', 10.^(skalay/20))
    set (gca, 'yticklabel', skalay)
    title ('Tschebyscheff Typ 1')
    xlabel ('Frequenz [kHz]')

subplot (144)
    [B, A] = ellip (2, 3, 40, fg*2*pi, 's');
    [H]    = freqs (B, A, W);
    loglog (W/2/pi/1000, abs (H), 'c-')
    hold on
    [B, A] = ellip (4, 3, 40, fg*2*pi, 's');
    [H]    = freqs (B, A, W);
    loglog (W/2/pi/1000, abs (H), 'b-')
    [B, A] = ellip (8, 3, 40, fg*2*pi, 's');
    [H]    = freqs (B, A, W);
    loglog (W/2/pi/1000, abs (H), 'k-')
    axis (axisskal)
    set (gca, 'xticklabel', skalax)
    set (gca, 'ytick', 10.^(skalay/20))
    set (gca, 'yticklabel', skalay)
    title ('Elliptisch (Cauer)')
    xlabel ('Frequenz [kHz]')

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Dititale Filter: FIR-Filter

fir1 (20, 0.2);               % Tiefpass, Durchlassbereich < 0.2 F_max
fir1 (20, 0.2, 'high');       % Hochpass, Durchlassbereich > 0.2 F_max
fir1 (20, [0.2 0.4]);         % Bandpass, Durchlassbereich 0.2 ... 0.4 F_max
fir1 (20, [0.2 0.4], 'stop'); % Bandsperre, Sperrbereich 0.2 ... 0.4 F_max

N   = 20;                           % Filterordnung (MATLAB-Zaehlung)
Ts  = 0.01;                         % Abtastfrequenz
fg  = 0.2;                          % Grenzfrequenz / halber Abtastfrequenz

B   = fir1 (N, fg, boxcar(N+1));    % Ohne Window
Bw  = fir1 (N, fg);                 % Hamming-Window

[H, F]  = freqz (B, 1, 100, 1/Ts);
[Hw, F] = freqz (Bw, 1, 100, 1/Ts);

figure
subplot (121)
    plot (-N/2:N/2, B, 'rx')
    hold on
    plot (-N/2:N/2, Bw, 'go')
    axis ([-N/2 N/2 -0.1 0.35])
    xlabel ('Nummer der Koeffizienten')
    title ('Filter-Koeffizienten')
    legend ('FIR', 'FIR + Hamming')
subplot (122)
    plot (F, abs (H), 'r--')
    hold on
    plot (F, abs (Hw), 'g-')
    axis ([0 0.5/Ts -0.4 1.4])
    xlabel ('Frequenz [Hz]')
    title ('Filter-bertragungsfunktion')
    legend ('FIR', 'FIR + Hamming')

%% filter, filtfilt

t = 0.01:0.01:1;                             % Zeitvektor (Fs = 100 Hz)
x = 5 + 8*sin(2*pi*8*t) + 4*cos(2*pi*33*t);  % Datenvektor ("Messwerte")

B = fir1 (20, 0.4);                          % Filterauslegung FIR
x_fir      = filter (B, 1, x);               % Filtern mit filter
x_filtfilt = filtfilt (B, 1, x);             % Filtern mit filtfilt
                                        
figure
    plot (t, x , 'k-')
    hold on
    plot (t, x_fir , 'r-')
    plot (t, x_filtfilt , 'b-')
    axis ([0 0.5 -10 30])
    xlabel ('Zeit [s]')
    title ('Diskrete Filterung mit FIR 20. Ordnung')
    legend ('ungefiltert', 'FIR filter', 'FIR filtfilt')

%% fir2

frequenz  = [0.0 0.3 0.3 0.6 1.0];              % Normierte Frequenzen
amplitude = [1.0 1.0 0.5 0.5 0.0];              % Amplituden

B = fir2 (50, frequenz, amplitude);             % FIR-Filter mit Ordnung 50
[H, F] = freqz (B, 1, 200, 100);                % bertragungsfunktion Fs = 100 Hz

figure
	plot (F, abs (H), 'r-')                     % Ist-bertragungsfunktion
	hold on
	plot (100/2*frequenz, amplitude, 'k-')      % Soll-bertragungsfunktion
	axis ([0 50 0 1.2])
	title ('FIR-Filterdesign mit fir2')
	xlabel ('Frequenz [Hz]')
	ylabel ('Amplitude')
    legend ('FIR2', 'Sollverlauf') 

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Digitale Filter: IIR-Filter

[B, A] = butter (6, 0.6);                            % Tiefpass, Grenzfrequenz 0.6*Fs/2
                                                     % Ausgabe: bertragungsfunktion
[z, p, k] = cheby1 (6, 3, 0.6, 'high');              % Hochpass, 3 dB Durchlass-Welligkeit
                                                     % Ausgabe: Nullst., Pole, Verstrkung
[a, b, c, d] = ellip (6, 3, 20, [0.3 0.6], 'stop');  % Bandsperre
                                                     % Ausgabe: Zustandsdarstellung
                                                  
t = 0.01:0.01:1;                                % Zeitvektor Fs = 100 Hz
x = 5 + 8*sin(2*pi*8*t) + 4*cos(2*pi*33*t);     % Beispiel-Signal

[B, A] = butter (2, 20/50);                     % Auslegung IIR-Filter
x_filt = filter (B, A, x);                      % Filtern mit filter
x_filtfilt = filtfilt (B, A, x);                % Filtern mit filtfilt

figure
    plot (t, x , 'k-')
    hold on
    plot (t, x_filt , 'r-')
    plot (t, x_filtfilt , 'b-')
    axis ([0 0.5 -10 30])
    xlabel ('Zeit [s]')
    title ('Diskrete Filterung mit IIR 2. Ordnung')
    legend ('ungefiltert', 'IIR filter', 'IIR filtfilt')

%% yulewalk

frequenz  = [0.0 0.3 0.3 0.6 1.0];              % Normierte Frequenzen
amplitude = [0.5 0.5 0.2 0.2 1.0];              % Amplituden

[B, A] = yulewalk (4, frequenz, amplitude);     % IIR-Filter mit Ordnung 4
[H, F] = freqz (B, A, 200, 100);                % bertragungsfunktion Fs = 100 Hz

figure
	plot (F, abs (H), 'r-')                     % Ist-bertragungsfunktion
	hold on
	plot (100/2*frequenz, amplitude, 'k-')      % Soll-bertragungsfunktion
	axis ([0 50 0 1.2])
	title ('IIR-Filterdesign mit yulewalk')
	xlabel ('Frequenz [Hz]')
	ylabel ('Amplitude')
    legend ('YULEWALK', 'Sollverlauf', 'Location', 'NW')   

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Filterdesign mit Prototyp-Tiefpssen

omega        = [40 80] * 2 * pi;                % Grenzfrequenz
[z, p, k]    = buttap  (4);                     % Prototyp-Tiefpass
[a, b, c, d] = zp2ss (z, p, k);                 % Umrechnung in Zustandsform
[a, b, c, d] = lp2bp (a, b, c, d, sqrt(prod(omega)), diff(omega));   % Bandpass
[B, A]       = ss2tf (a, b, c, d);              % Analoges Filter

%% Beispiel: Filter mit aperiodischem Einschwingen

ordnung = 4;             % Ordnung des Filters
z = [];                  % Keine Nullstellen
p = -ones (ordnung, 1);  % Polstellen bei s=-1 entsprechend der Filterordnung
k = 1;                   % Verstrkung 1

omega        = 100 * 2 * pi;                    % Grenzfrequenz 100 Hz
korrektur    = 1 / sqrt (2 ^(1 / ordnung) -1);  % Korrekturfaktor > 1
[a, b, c, d] = zp2ss (z, p, k);                 % Umrechnung in Zustandsform
[a, b, c, d] = lp2lp (a, b, c, d, korrektur);   % Korrektur
[a, b, c, d] = lp2lp (a, b, c, d, omega);       % Transformation
[B, A]       = ss2tf (a, b, c, d);              % bertragungsfunktion

[H, W]       = freqs (B, A, 2*pi*logspace(1,3));% Frequenzgang analog
system       = tf (B, A);                       % bertragungsfunktion als Objekt
[Y, T]       = step (system, 0.02);             % Sprungantwort

figure
subplot (121)
    loglog (W/(2*pi), abs (H))
    hold on
    plot ([10 1000], [1 1]/sqrt(2), 'k:')
    axis ([10 1000 0.001 2])
    title ('bertragungsfunktion')
    skalax   = [10 100 1000];
    skalay   = -40:20:20;
    set (gca, 'xticklabel', skalax)
    set (gca, 'ytick', 10.^(skalay/20))
    set (gca, 'yticklabel', skalay)
    legend ('bertragungsfunktion', '-3 dB', 'Location', 'SW')
    xlabel ('Frequenz [Hz]')
    ylabel ('Amplitude [dB]')
subplot (122)
    plot (T*1000, Y, '-')
    axis ([0 20 0 1.1])
    title ('Sprungantwort')
    xlabel ('Zeit [ms]')
    ylabel ('Amplitude')

%% Beispiel: Pre-warping zu obigem Filter

[H, W]       = freqs (B, A);                    % Frequenzgang analog

Fs           = 300;                             % Abtastfrequenz 250 Hz
[e, f, g, h] = bilinear (a, b, c, d, Fs);       % Digitales Filter ohne Pre-warping
[Bd, Ad]     = ss2tf (e, f, g, h);              % bertragungsfunktion
[Hd, Fd]     = freqz (Bd, Ad, 256, Fs);         % Frequenzgang ohne Pre-Warping

[e, f, g, h] = bilinear (a, b, c, d, Fs, 100);  % Digitales Filter ohne Pre-warping
[Bp, Ap]     = ss2tf (e, f, g, h);              % bertragungsfunktion
[Hp, Fp]     = freqz (Bp, Ap, 256, Fs);         % Frequenzgang mit Pre-Warping

figure
    plot (W/(2*pi), abs (H), 'g-')
    hold on
    plot (Fd, abs (Hd), 'r-')
    plot (Fp, abs (Hp), 'b-')
    plot ([1 Fs/2], [1 1]/sqrt(2), 'k:')
    axis ([0 Fs/2 0 1.2])
    title ('Pre-warping mit "bilinear"')
    xlabel ('Frequenz [Hz]')
    ylabel ('Amplitude')
    legend ('Analog', 'Digital', 'Pre-warping', '-3 dB', 'Location', 'SW')

%% Beispiel: Pre-warping zu Bandpass

omega        = [40 80] * 2 * pi;                % Grenzfrequenz
[z, p, k]    = buttap (4);                      % Prototyp-Tiefpass
[a, b, c, d] = zp2ss (z, p, k);                 % Umrechnung in Zustandsform
[a, b, c, d] = lp2bp (a, b, c, d, sqrt(prod(omega)), diff(omega));   % Bandpass
[B, A]       = ss2tf (a, b, c, d);              % Analoges Filter
[H, W]       = freqs (B, A);                    % Frequenzgang analog

Fs           = 200;                             % Abtastfrequenz 200 Hz
[e, f, g, h] = bilinear (a, b, c, d, Fs);       % Digitales Filter ohne Pre-warping
[Bd, Ad]     = ss2tf (e, f, g, h);              % bertragungsfunktion
[Hd, Fd]     = freqz (Bd, Ad, 256, Fs);         % Frequenzgang ohne Pre-Warping

omega        = [40 80] * 2 * pi;                % Grenzfrequenz
omega        = 2 * Fs * tan (omega / Fs / 2);   % Grenzfrequenz mit Pre-warping
[z, p, k]    = buttap  (4);                     % Prototyp-Tiefpass
[a, b, c, d] = zp2ss (z, p, k);                 % Umrechnung in Zustandsform
[a, b, c, d] = lp2bp (a, b, c, d, sqrt(prod(omega)), diff(omega));   % Bandpass
[e, f, g, h] = bilinear (a, b, c, d, Fs);       % Dig. Filter mit Pre-warping
[Bp, Ap]     = ss2tf (e, f, g, h);              % bertragungsfunktion
[Hp, Fp]     = freqz (Bp, Ap, 256, Fs);         % Frequenzgang mit Pre-Warping

figure
    plot (W/(2*pi), abs (H), 'g-')
    hold on
    plot (Fd, abs (Hd), 'r-')
    plot (Fp, abs (Hp), 'b-')
    plot ([1 Fs/2], [1 1]/sqrt(2), 'k:')
    axis ([0 Fs/2 0 1.2])
    title ('Pre-warping manuell')
    xlabel ('Frequenz [Hz]')
    ylabel ('Amplitude')
    legend ('Analog', 'Digital', 'Pre-warping', '-3 dB', 'Location', 'NW')

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
